@@ -6,7 +6,6 @@ import functools
 import subprocess
 import time
 import traceback
-
 from abc import ABC, abstractmethod
 from typing import Any, Callable, Dict, List, Optional
 
@@ -14,6 +13,7 @@ from packageurl import PackageURL  # type: ignore
 
 from hoppr import plugin_utils
 from hoppr.context import Context
+from hoppr.hoppr_types.bom_access import BomAccess
 from hoppr.hoppr_types.component_coverage import ComponentCoverage
 from hoppr.mem_logger import MemoryLogger
 from hoppr.result import Result
@@ -187,6 +187,7 @@ class HopprPlugin(ABC):
     required_commands: List[str] = []
     supported_purl_types: List[str] = []
     default_component_coverage = ComponentCoverage.OPTIONAL
+    bom_access = BomAccess.NO_ACCESS
 
     def __init__(
         self,
