@@ -54,10 +54,12 @@ class SBOMRef:
 
     url: Optional[str]
     local: Optional[str]
+    oci: Optional[str]
 
-    def __init__(self, url: str = None, local: str = None) -> None:
+    def __init__(self, url: str = None, local: str = None, oci: str = None) -> None:
         self.url = url
         self.local = local
+        self.oci = oci
 
     def __hash__(self):
         """
