@@ -2,34 +2,39 @@
 Run Hoppr processing, using multiple processors
 """
 from __future__ import annotations
+from copy import deepcopy
 
-import os
 import logging
+import os
 import platform
 import shutil
 import socket
 import tempfile
 import time
-
 from concurrent.futures import Future, ProcessPoolExecutor
 from datetime import datetime
 from itertools import zip_longest
 from multiprocessing import cpu_count
 from os import PathLike
 from pathlib import Path
-from typing import Dict, List, Optional, Tuple, Union
+from typing import Any, Dict, List, Optional, Tuple, Union
 from urllib.parse import quote_plus
 
-from hoppr_cyclonedx_models.cyclonedx_1_4 import Component  # type: ignore
+from hoppr_cyclonedx_models.cyclonedx_1_4 import (
+    Component,
+    CyclonedxSoftwareBillOfMaterialsStandard as Bom,
+)  # type: ignore
 from packageurl import PackageURL  # type: ignore
 from typer import colors, echo, secho
 
+from hoppr import oci_artifacts
 from hoppr.base_plugins.hoppr import HopprPlugin
 from hoppr.configs.manifest import Manifest
 from hoppr.configs.transfer import Transfer
 from hoppr.context import Context
 from hoppr.exceptions import HopprPluginError
 from hoppr.flatten_sboms import flatten_sboms
+from hoppr.hoppr_types.bom_access import BomAccess
 from hoppr.hoppr_types.component_coverage import ComponentCoverage
 from hoppr.hoppr_types.transfer_file_content import Stage as StageRef
 from hoppr.mem_logger import MemoryLogger
@@ -56,6 +61,13 @@ def _run_plugin(
     else:
         result = Result.fail(f"Invalid method: {method_name}")
 
+    if result.return_obj is not None:
+        if not plugin.bom_access.has_access_to(result.return_obj):
+            result = Result.fail(
+                f"Plugin {plugin.__class__.__name__} has BOM access level {plugin.bom_access.name}, "
+                + f"but returned an object of type {type(result.return_obj).__name__}"
+            )
+
     return result
 
 
@@ -70,9 +82,12 @@ class StageProcessor:  # pylint: disable=too-few-public-methods
         self.stage_id = stage_ref.name
         self.context = context
         self.plugin_ref_list = stage_ref.plugins
-        self.component_coverage = None
+        self.config_component_coverage = None
         if stage_ref.component_coverage is not None:
-            self.component_coverage = ComponentCoverage[stage_ref.component_coverage]
+            self.config_component_coverage = ComponentCoverage[
+                stage_ref.component_coverage
+            ]
+        self.required_coverage = ComponentCoverage.OPTIONAL
         self.plugins: List[HopprPlugin] = []
         self.results: Dict = {}
 
@@ -83,10 +98,13 @@ class StageProcessor:  # pylint: disable=too-few-public-methods
 
         try:
             self.plugins = self._load_plugins()
+            self.required_coverage = self._get_required_coverage()
         except (ModuleNotFoundError, HopprPluginError) as err:
             return Result.fail(str(err))
 
-        result = Result.success()
+        result = self._check_bom_access()
+        if not result.is_success():
+            return result
 
         # Run each sub-stage (pre_stage_process, process_component, post_stage_process)
         # for all plugins (and, for process_component, for all components).
@@ -115,7 +133,7 @@ class StageProcessor:  # pylint: disable=too-few-public-methods
 
                     # Create one concurrent future object to run this method for each component
 
-                    for component in self.context.consolidated_sbom.components or []:
+                    for component in self.context.delivered_sbom.components or []:
                         future_proc = executor.submit(
                             _run_plugin, plugin, method_name, component
                         )
@@ -148,6 +166,7 @@ class StageProcessor:  # pylint: disable=too-few-public-methods
                         method_name, plugin.__class__.__name__, future_result, comp
                     )
                     self._report_result(plugin.__class__.__name__, comp, future_result)
+                    self._update_bom(future_result.return_obj, comp)
 
                     if future_result.is_fail():
                         failures += 1
@@ -173,8 +192,8 @@ class StageProcessor:  # pylint: disable=too-few-public-methods
         )
 
     def _get_required_coverage(self) -> ComponentCoverage:
-        if self.component_coverage is not None:
-            return self.component_coverage
+        if self.config_component_coverage is not None:
+            return self.config_component_coverage
 
         if len(self.plugins) == 0:
             return ComponentCoverage.OPTIONAL
@@ -191,17 +210,16 @@ class StageProcessor:  # pylint: disable=too-few-public-methods
         return coverage
 
     def _check_component_coverage(self, method_name: str) -> int:
-        required_coverage = self._get_required_coverage()
         result_count: Dict[Optional[str], int] = {}
         for (_, purl, _) in self.results.get(method_name, []):
             result_count[purl] = result_count.get(purl, 0) + 1
 
         additional_failures = 0
-        for component in self.context.consolidated_sbom.components or []:
+        for component in self.context.delivered_sbom.components or []:
             count = result_count.get(component.purl, 0)
-            if not required_coverage.accepts_count(count):
+            if not self.required_coverage.accepts_count(count):
                 bad_comp_result = Result.fail(
-                    f"Component processed {count} times, {required_coverage.name} coverage required"
+                    f"Component processed {count} times, {self.required_coverage.name} coverage required"
                 )
                 self._save_result(
                     method_name, f"Stage {self.stage_id}", bad_comp_result, component
@@ -213,6 +231,54 @@ class StageProcessor:  # pylint: disable=too-few-public-methods
 
         return additional_failures
 
+    def _check_bom_access(self) -> Result:
+        access_counts: Dict[BomAccess, list[str]] = {}
+        for access in BomAccess:
+            access_counts[access] = []
+
+        for plugin in self.plugins:
+            access_counts[plugin.bom_access].append(plugin.__class__.__name__)
+
+        if len(access_counts[BomAccess.FULL_ACCESS]) > 0 and len(self.plugins) > 1:
+            msg = (
+                f"Stage {self.stage_id} has one or more plugins with {BomAccess.FULL_ACCESS.name}: "
+                + ", ".join(access_counts[BomAccess.FULL_ACCESS])
+                + ", and multiple plugins defined for the stage."
+                + "\n    Any plugin with FULL BOM access must be the only plugin in the stage"
+            )
+            secho(msg, fg=colors.RED)
+            return Result.fail(msg)
+
+        if (
+            len(access_counts[BomAccess.COMPONENT_ACCESS]) > 0
+            and self.required_coverage.max_value > 1
+        ):
+            msg = (
+                f"Stage {self.stage_id} has one or more plugins with {BomAccess.COMPONENT_ACCESS.name}: "
+                + ", ".join(access_counts[BomAccess.COMPONENT_ACCESS])
+                + f", and required component coverage for the stage of {self.required_coverage.name}."
+                + "\n    If any plugins have COMPONENT access, the stage required coverage must be "
+                + "EXACTLY_ONCE or NO_MORE_THAN_ONCE."
+            )
+            secho(msg, fg=colors.RED)
+            return Result.fail(msg)
+
+        return Result.success()
+
+    def _update_bom(self, return_obj: Any, comp: Component | None):
+        if self.context.delivered_sbom.components is not None and isinstance(
+            return_obj, Component
+        ):
+            for index, delivered_comp in enumerate(
+                self.context.delivered_sbom.components
+            ):
+                if delivered_comp == comp:
+                    self.context.delivered_sbom.components[index] = return_obj
+                    break
+
+        elif isinstance(return_obj, Bom):
+            self.context.delivered_sbom = return_obj
+
     @staticmethod
     def _report_result(plugin, comp, result):
         desc = f"      {plugin} {result.status.name}"
@@ -250,12 +316,24 @@ class StageProcessor:  # pylint: disable=too-few-public-methods
         """
         Create a list consisting of a single instance of each plug-in class used in this stage
         """
+
+        used_purl_types = set()
+
+        for comp in self.context.delivered_sbom.components or []:
+            purl_type = PackageURL.from_string(comp.purl).type
+            used_purl_types.add(purl_type)
+
         plugin_list = []
         for plugin_ref in self.plugin_ref_list:
             plugin_inst = plugin_instance(
                 plugin_ref.name, self.context, plugin_ref.config
             )
-            plugin_list.append(plugin_inst)
+            if (
+                plugin_inst.supported_purl_types == []
+                or len(used_purl_types.intersection(plugin_inst.supported_purl_types))
+                > 0
+            ):
+                plugin_list.append(plugin_inst)
 
         return plugin_list
 
@@ -301,6 +379,15 @@ class HopprProcessor:  # pylint: disable=too-few-public-methods
         target = Path(target_dir, quote_plus(f"{url}"))
         download_file(url, f"{target}", None)
 
+    def _collect_oci_artifact(
+        self, artifact: str, target_dir: Union[str, PathLike[str]]
+    ):
+        target = os.path.join(target_dir, quote_plus(artifact))
+        self.get_logger().info(f"Collecting metadata from OCI {artifact} to {target}")
+        return oci_artifacts.pull_artifact_to_disk(
+            artifact, target, allow_version_discovery=True
+        )
+
     def _collect_manifest_metadata(
         self, manifest: Manifest, target_dir: Union[str, PathLike[str]]
     ) -> None:
@@ -308,7 +395,10 @@ class HopprProcessor:  # pylint: disable=too-few-public-methods
             manifest.manifest_file_content.includes, manifest.children
         ):
             if child_ref.local is not None:
-                self._collect_file(child_ref.local, target_dir)
+                child_path = Path(manifest.manifest_location).parent.joinpath(
+                    child_ref.local
+                )
+                self._collect_file(child_path, target_dir)
             else:
                 self._collect_url(f"{child_ref.url}", Path(target_dir))
 
@@ -316,7 +406,12 @@ class HopprProcessor:  # pylint: disable=too-few-public-methods
 
         for bom_ref in manifest.manifest_file_content.sbom_refs:
             if bom_ref.local is not None:
-                self._collect_file(bom_ref.local, target_dir)
+                sbom_file_path = Path(manifest.manifest_location).parent.joinpath(
+                    bom_ref.local
+                )
+                self._collect_file(sbom_file_path, target_dir)
+            elif bom_ref.oci is not None:
+                self._collect_oci_artifact(bom_ref.oci, target_dir)
             else:
                 self._collect_url(f"{bom_ref.url}", target_dir)
 
@@ -353,6 +448,11 @@ class HopprProcessor:  # pylint: disable=too-few-public-methods
         ) as bom_data:
             bom_data.write(self.context.consolidated_sbom.json(by_alias=True, indent=2))
 
+        with open(
+            target_dir.joinpath("_delivered_bom.json"), "w", encoding="utf-8"
+        ) as bom_data:
+            bom_data.write(self.context.delivered_sbom.json(by_alias=True, indent=2))
+
     def _summarize_results(self) -> int:
         """
         Summarize the results of a HopprProcess run
@@ -401,10 +501,17 @@ class HopprProcessor:  # pylint: disable=too-few-public-methods
 
         with tempfile.TemporaryDirectory() as collection_root:
 
+            flattened_bom = flatten_sboms(self.manifest)
+            if len(flattened_bom.components or []) == 0:
+                msg = "No SBOMs defined in manifests, or SBOMs contain no components.  Nothing to process."
+                secho(msg, fg=colors.RED)
+                return Result.fail(msg)
+
             self.context = Context(
                 manifest=self.manifest,
                 collect_root_dir=collection_root,
-                consolidated_sbom=flatten_sboms(self.manifest),
+                consolidated_sbom=flattened_bom,
+                delivered_sbom=deepcopy(flattened_bom),
                 max_processes=cpu_count(),
                 log_level=self.log_level,
             )
