@@ -3,9 +3,12 @@ Class to store results of processes
 """
 
 import enum
+from typing import Any
 
 import requests
 
+from hoppr.exceptions import HopprError
+
 
 class ResultStatus(enum.IntEnum):
     """
@@ -23,9 +26,10 @@ class Result:
     Class to store results of processes
     """
 
-    def __init__(self, status: ResultStatus, message: str = ""):
+    def __init__(self, status: ResultStatus, message: str = "", return_obj: Any = None):
         self.status = status
         self.message = message
+        self.return_obj = return_obj
 
     def __str__(self):
         result_msg = f"{self.status.name}"
@@ -35,32 +39,32 @@ class Result:
         return result_msg.rstrip()
 
     @staticmethod
-    def success(message: str = "") -> "Result":
+    def success(message="", return_obj: Any = None) -> "Result":
         """
         Convenience method for generating success messages
         """
-        return Result(ResultStatus.SUCCESS, message)
+        return Result(ResultStatus.SUCCESS, message, return_obj)
 
     @staticmethod
-    def retry(message: str = "") -> "Result":
+    def retry(message="", return_obj: Any = None) -> "Result":
         """
         Convenience method for generating retry messages
         """
-        return Result(ResultStatus.RETRY, message)
+        return Result(ResultStatus.RETRY, message, return_obj)
 
     @staticmethod
-    def fail(message: str = "") -> "Result":
+    def fail(message="", return_obj: Any = None) -> "Result":
         """
         Convenience method for generating failure messages
         """
-        return Result(ResultStatus.FAIL, message)
+        return Result(ResultStatus.FAIL, message, return_obj)
 
     @staticmethod
-    def skip(message: str = "") -> "Result":
+    def skip(message="", return_obj: Any = None) -> "Result":
         """
         Convenience method for generating skip messages
         """
-        return Result(ResultStatus.SKIP, message)
+        return Result(ResultStatus.SKIP, message, return_obj)
 
     def is_success(self) -> bool:
         """
@@ -99,25 +103,28 @@ class Result:
         else:
             self.message += "\n" + other.message
 
+        if self.return_obj is None:
+            self.return_obj = other.return_obj
+        elif other.return_obj is not None:
+            raise HopprError("Unable to merge two results with return objects")
+
         return
 
     @staticmethod
-    def from_http_response(response: requests.Response):
+    def from_http_response(response: requests.Response, return_obj: Any = None):
         """
         Build a Result object from an HTTP request response
         """
 
         match response.status_code:
             case response_code if 200 <= response_code <= 299:  # pylint: disable=used-before-assignment
-                status = ResultStatus.SUCCESS
+                return Result.success(
+                    f"HTTP Status Code: {response.status_code}", return_obj=return_obj
+                )
             case response_code if 500 <= response_code:
                 status = ResultStatus.RETRY
             case _:
                 status = ResultStatus.FAIL
 
-        message = f"HTTP Status Code: {response.status_code}"
-
-        if status.name in ["RETRY", "FAIL"]:
-            message = f"{message}; {response.reason or response.text}"
-
-        return Result(status, message)
+        message = f"HTTP Status Code: {response.status_code}; {response.reason or response.text}"
+        return Result(status, message, return_obj=return_obj)
