@@ -5,10 +5,9 @@ Hoppr utility functions
 import importlib
 import inspect
 import json
-
 from json.decoder import JSONDecodeError
 from pathlib import Path
-from typing import Any, List, Set
+from typing import Any, Dict, List, Set
 
 import yaml
 from yaml.parser import ParserError as YamlParserError
@@ -69,7 +68,7 @@ def plugin_instance(plugin_name, context: Context, config: Any = None):
     return plugin_obj(context=context, config=config)
 
 
-def load_string(contents: str):
+def load_string(contents: str) -> Dict:
     """
     Try loading yaml, json, and xml files into a dict.
     """
@@ -92,7 +91,7 @@ def load_string(contents: str):
     raise HopprLoadDataError("Unable to recognize data as either json or yaml")
 
 
-def load_file(input_file_path: Path):
+def load_file(input_file_path: Path) -> Dict:
     """
     Load file content (either json or yml) into a dict
     """
@@ -100,7 +99,7 @@ def load_file(input_file_path: Path):
     if not input_file_path.is_file():
         raise HopprLoadDataError(f"{input_file_path} is not a file, cannot be opened.")
 
-    with open(input_file_path, "r", encoding="utf-8") as input_file:
+    with input_file_path.open(mode="r", encoding="utf-8") as input_file:
         content = input_file.read()
         return load_string(content)
 
